/**
 * OW2 Util
 * Copyright (C) 2008 
 * Contact: easybeans@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: XmlFileDeployableFactoryImpl.java
* --------------------------------------------------------------------------
*/

package org.ow2.util.plan.deploy.deployable.impl.factory;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.ow2.util.archive.api.IFileArchive;
import org.ow2.util.plan.deploy.deployable.api.factory.FileDeployableException;
import org.ow2.util.plan.deploy.deployable.api.factory.XmlFileDeployableFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.deploy.deployable.api.FileDeployable;
import org.ow2.util.plan.deploy.deployable.internal.xmlhelper.XmlHelper;
import org.xml.sax.SAXException;

/**
 * Implementation for the XmlFileDeployableFactory interface.
 * @author Mickaël LEDUQUE
 */
public class XmlFileDeployableFactoryImpl implements XmlFileDeployableFactory {

    /**
     * The logger.
     */
    private Log logger = LogFactory.getLog(XmlFileDeployableFactoryImpl.class);

    /**
     * The XmlHelper used to identify XML files.
     */
    private XmlHelper xmlHelper = new XmlHelper();

    /**
     * Default constructor.
     * @throws ParserConfigurationException when the XmlHelper initilization
     * fails.
     * @throws SAXException when the XmlHelper initilization
     * fails.
     */
    public XmlFileDeployableFactoryImpl()
                    throws ParserConfigurationException, SAXException {
    /* Needed because xmlHelper initialization can throw exceptions */
    }

    /**
     * XML namespace to FileDeployable association.
     */
    private Map<String, Class<? extends FileDeployable<?, ?>>> xmlNSToDeployableMap = new HashMap<String, Class<? extends FileDeployable<?, ?>>>();

    /**
     * Returns the file deployable constructed from the IFileArchive using
     * the given namespace.
     * @param fileArchive the archive that must be converted.
     * @param nameSpace the namespace to be used.
     * @return the FileDeployable.
     * @throws FileDeployableException when the Deployable could not be constructed.
     */
    private FileDeployable<?, ?> getXmlFileDeployable(final IFileArchive fileArchive,
            final String nameSpace) throws FileDeployableException {
        Class<? extends FileDeployable<?, ?>> clazz = xmlNSToDeployableMap.get(nameSpace);
        if (clazz == null) {
            logger.info("No namespace match found for {0}", nameSpace);
            throw new FileDeployableException();
        }
        Constructor<? extends FileDeployable<?, ?>> constructor = null;
        try {
            constructor = clazz.getDeclaredConstructor(IFileArchive.class);
        } catch (NoSuchMethodException e) {
            logger.error(
                            "No {0}(IArchive archive) constructor; aborting deployable generation",
                            clazz.getName());
            throw new FileDeployableException(e);
        }
        try {
            return constructor.newInstance(fileArchive);
        } catch (InvocationTargetException e) {
            /*
             * Catch the InvocationTargetException because the nested exception
             * is more meaningful.
             */
            Throwable nested = e.getCause();
            throw new FileDeployableException(nested);
        } catch (Exception e) {
            logger.error("Caught error on constructor invokation : {0}", e);
            throw new FileDeployableException();
        }
    }

    /**
     * {@inheritDoc}
     */
    public FileDeployable<?, ?> getFileDeployable(final IFileArchive fileArchive) throws FileDeployableException {
        String namespace = null;
        try {
            namespace = xmlHelper.getRootElementNameSpace(fileArchive.getFile());
        } catch (IOException e) {
            logger.error("IO Exception while parsing file {0}", fileArchive.getFile(), e);
            throw new FileDeployableException(e);
        } catch (SAXException e) {
            logger.error("Exception while parsing file {0} : this is not a XML file", fileArchive.getFile(), e);
            throw new FileDeployableException(e);
        }

        return getXmlFileDeployable(fileArchive, namespace);
    }

    /**
     * {@inheritDoc}
     */
    public void registerFileDeployable(final Class<? extends FileDeployable<?, ?>> clazz,
            final String nameSpace) {
        xmlNSToDeployableMap.put(nameSpace, clazz);
    }

    /**
     * {@inheritDoc}
     */
    public void unregisterFileDeployable(final String nameSpace) {
        xmlNSToDeployableMap.remove(nameSpace);

    }

}
